==> dbnd_snowflake-0.29.0-py2.py3-none-any.whl: dbnd_snowflake/airflow_operators.py <==
# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# Based on SnoflakeOperator
from typing import Optional

from airflow.contrib.hooks.snowflake_hook import SnowflakeHook
from airflow.contrib.operators.snowflake_operator import SnowflakeOperator
from airflow.utils.decorators import apply_defaults

from dbnd._core.commands.metrics import log_snowflake_table
from dbnd_snowflake import log_snowflake_resource_usage


class LogSnowflakeTableOperator(SnowflakeOperator):
    """
    Queries Snowflake database for a table preview and metadata such as data schema
    and shape and tracks it into DBND

    :param str table: name of a table we want to track
    :param str key: Optional key for dbnd metrics. Defaults to table name
    :param with_preview: Set `True` to track table preview (first 20 rows)
    :param with_schema: Set `True` to track table schema, i.e. columns names, types and table dimensions
    :param str snowflake_conn_id: reference to specific snowflake connection id
    :param str warehouse: name of warehouse (will overwrite any warehouse
        defined in the connection's extra JSON)
    :param str database: name of database (will overwrite database defined
        in connection)
    :param str schema: name of schema (will overwrite schema defined in
        connection)
    :param str role: name of role (will overwrite any role defined in
        connection's extra JSON)
    """

    @apply_defaults
    def __init__(
        self,
        table,
        key=None,
        account=None,
        with_preview: Optional[bool] = None,
        with_schema: Optional[bool] = None,
        *args,
        **kwargs
    ):
        super(LogSnowflakeTableOperator, self).__init__(*args, sql=None, **kwargs)

        self.table = table
        self.key = key
        self.with_preview = with_preview
        self.with_schema = with_schema

        # TODO: deprecate
        self.account = account

    def get_hook(self):
        return SnowflakeHook(
            snowflake_conn_id=self.snowflake_conn_id,
            account=self.account,
            warehouse=self.warehouse,
            database=self.database,
            role=self.role,
            schema=self.schema,
        )

    def execute(self, context):
        self.log.info("Executing: %s", self.sql)
        hook = self.get_hook()

        connection_string = hook.get_uri()
        return log_snowflake_table(
            self.table,
            connection_string,
            hook.database,
            hook.schema,
            key=self.key,
            with_preview=self.with_preview,
            with_schema=self.with_schema,
        )


def log_snowflake_operator(
    op: SnowflakeOperator,
    table: str,
    with_preview: Optional[bool] = None,
    with_schema: Optional[bool] = None,
    **kwargs
):
    task_id = kwargs.pop("task_id", "log_table_%s_%s" % (op.task_id, table))
    return LogSnowflakeTableOperator(
        table=table,
        snowflake_conn_id=kwargs.pop("snowflake_conn_id", op.snowflake_conn_id),
        database=kwargs.pop("database", op.database),
        warehouse=kwargs.pop("warehouse", op.warehouse),
        role=kwargs.pop("role", op.role),
        schema=kwargs.pop("schema", op.schema),
        task_id=task_id,
        with_preview=with_preview,
        with_schema=with_schema,
        **kwargs
    )


class LogSnowflakeResourceOperator(SnowflakeOperator):
    """
    Queries Snowflake database for resources used to execute specific SQL query and tracks it into DBND

    :param str snowflake_conn_id: reference to specific snowflake connection id
    :param str session_id: Optional id of a Snowflake session used to execute query.
        Can be used if the same query is executed
    :param str key: Optional key for dbnd metrics. Defaults to table name
    :param sql: Exact sql query executed previously.
    :param str warehouse: name of warehouse (will overwrite any warehouse
        defined in the connection's extra JSON)
    :param str database: name of database (will overwrite database defined
        in connection)
    :param str schema: name of schema (will overwrite schema defined in
        connection)
    :param str role: name of role (will overwrite any role defined in
        connection's extra JSON)
    """

    @apply_defaults
    def __init__(self, session_id=None, key=None, account=None, *args, **kwargs):
        super(LogSnowflakeResourceOperator, self).__init__(*args, **kwargs)

        self.session_id = session_id
        self.key = key
        # TODO: deprecate
        self.account = account

    def get_hook(self):
        return SnowflakeHook(
            snowflake_conn_id=self.snowflake_conn_id,
            account=self.account,
            warehouse=self.warehouse,
            database=self.database,
            role=self.role,
            schema=self.schema,
        )

    def execute(self, context):
        self.log.info("Executing: %s", self.sql)
        hook = self.get_hook()

        conn_params = hook._get_conn_params()
        user = conn_params["user"]

        log_snowflake_resource_usage(
            self.sql,
            database=hook.database,
            user=user,
            connection_string=hook.get_uri(),
            session_id=self.session_id,
            key=self.key,
        )


def log_snowflake_resource_operator(op: SnowflakeOperator, **kwargs):
    task_id = kwargs.pop("task_id", "log_resources_%s" % (op.task_id))
    return LogSnowflakeResourceOperator(
        sql=op.sql,
        snowflake_conn_id=kwargs.pop("snowflake_conn_id", op.snowflake_conn_id),
        database=kwargs.pop("database", op.database),
        warehouse=kwargs.pop("warehouse", op.warehouse),
        role=kwargs.pop("role", op.role),
        schema=kwargs.pop("schema", op.schema),
        task_id=task_id,
        **kwargs
    )

==> dbnd_snowflake-0.29.6-py2.py3-none-any.whl: dbnd_snowflake/airflow_operators.py <==
# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# Based on SnoflakeOperator
import logging

from typing import Optional

from airflow.contrib.hooks.snowflake_hook import SnowflakeHook
from airflow.contrib.operators.snowflake_operator import SnowflakeOperator
from airflow.utils.decorators import apply_defaults

from dbnd_snowflake import log_snowflake_resource_usage, log_snowflake_table


logger = logging.getLogger(__name__)


class LogSnowflakeTableOperator(SnowflakeOperator):
    """
    Queries Snowflake database for a table preview and metadata such as data schema
    and shape and tracks it into DBND

    :param str table: name of a table we want to track
    :param str key: Optional key for dbnd metrics. Defaults to table name
    :param with_preview: Set `True` to track table preview (first 20 rows)
    :param with_schema: Set `True` to track table schema, i.e. columns names, types and table dimensions
    :param str snowflake_conn_id: reference to specific snowflake connection id
    :param str warehouse: name of warehouse (will overwrite any warehouse
        defined in the connection's extra JSON)
    :param str database: name of database (will overwrite database defined
        in connection)
    :param str schema: name of schema (will overwrite schema defined in
        connection)
    :param str role: name of role (will overwrite any role defined in
        connection's extra JSON)
    """

    template_fields = ("table", "key")
    ui_color = "#2B7BB3"

    @apply_defaults
    def __init__(
        self,
        table,
        key=None,
        account=None,
        with_preview: Optional[bool] = None,
        with_schema: Optional[bool] = None,
        raise_on_error: bool = False,
        *args,
        **kwargs
    ):
        super(LogSnowflakeTableOperator, self).__init__(*args, sql=None, **kwargs)

        self.table = table
        self.key = key
        self.with_preview = with_preview
        self.with_schema = with_schema
        self.raise_on_error = raise_on_error

        # TODO: deprecate
        self.account = account

    def get_hook(self):
        return SnowflakeHook(
            snowflake_conn_id=self.snowflake_conn_id,
            account=self.account,
            warehouse=self.warehouse,
            database=self.database,
            role=self.role,
            schema=self.schema,
        )

    def execute(self, context):
        self.log.info("Executing: %s", self.sql)
        hook = self.get_hook()

        connection_string = hook.get_uri()
        return log_snowflake_table(
            table_name=self.table,
            connection_string=connection_string,
            database=hook.database,
            schema=hook.schema,
            key=self.key,
            with_preview=self.with_preview,
            with_schema=self.with_schema,
            raise_on_error=self.raise_on_error,
        )


def log_snowflake_operator(
    op: SnowflakeOperator,
    table: str,
    with_preview: Optional[bool] = None,
    with_schema: Optional[bool] = None,
    raise_on_error: bool = False,
    **kwargs
):
    task_id = kwargs.pop("task_id", "log_table_%s_%s" % (op.task_id, table))
    return LogSnowflakeTableOperator(
        table=table,
        snowflake_conn_id=kwargs.pop("snowflake_conn_id", op.snowflake_conn_id),
        database=kwargs.pop("database", op.database),
        warehouse=kwargs.pop("warehouse", op.warehouse),
        role=kwargs.pop("role", op.role),
        schema=kwargs.pop("schema", op.schema),
        task_id=task_id,
        with_preview=with_preview,
        with_schema=with_schema,
        raise_on_error=raise_on_error,
        **kwargs
    )


class LogSnowflakeResourceOperator(SnowflakeOperator):
    """
    Queries Snowflake database for resources used to execute specific SQL query and tracks it into DBND

    :param str snowflake_conn_id: reference to specific snowflake connection id
    :param session_id: Optional id of a Snowflake session used to execute query.
    :param query_id: Snowflake's QUERY_ID of a query executed previously you want to find and log resources.
        Use `run_snowflake()` helper to execute query and get it's session_id & query_id
    :param key: Optional key for dbnd metrics. Defaults to table name
    :param history_window: How deep to search in QUERY_HISTORY for a query. Set in minutes.
    :param retries: How deep to search in QUERY_HISTORY for a query. Set in minutes.
    :param retry_pause: How deep to search in QUERY_HISTORY for a query. Set in minutes.
    :param sql: Exact sql query executed previously.
    :param str warehouse: name of warehouse (will overwrite any warehouse
        defined in the connection's extra JSON)
    :param str database: name of database (will overwrite database defined
        in connection)
    :param str schema: name of schema (will overwrite schema defined in
        connection)
    :param str role: name of role (will overwrite any role defined in
        connection's extra JSON)
    """

    template_fields = ("sql", "key")
    ui_color = "#2B7BB3"

    @apply_defaults
    def __init__(
        self,
        session_id: Optional[int] = None,
        query_id: Optional[str] = None,
        key: Optional[str] = None,
        account: Optional[str] = None,
        history_window: float = 15,
        query_history_result_limit: int = 100,
        retries: int = 3,
        retry_pause: int = 15,
        raise_on_error: bool = False,
        *args,
        **kwargs
    ):
        super(LogSnowflakeResourceOperator, self).__init__(*args, **kwargs)

        self.session_id = session_id
        self.query_id = query_id
        self.key = key
        self.history_window = history_window
        self.query_history_result_limit = query_history_result_limit
        self.retries = retries
        self.retry_pause = retry_pause
        self.raise_on_error = raise_on_error
        # TODO: deprecate
        self.account = account

    def get_hook(self):
        return SnowflakeHook(
            snowflake_conn_id=self.snowflake_conn_id,
            account=self.account,
            warehouse=self.warehouse,
            database=self.database,
            role=self.role,
            schema=self.schema,
        )

    def execute(self, context):
        from dbnd_snowflake.airflow_hooks import snowflake_run

        self.log.info("Executing: %s", self.sql)
        hook = self.get_hook()

        conn_params = hook._get_conn_params()

        session_id, query_ids = snowflake_run(hook, self.sql)
        user = conn_params["user"]

        log_snowflake_resource_usage(
            database=hook.database,
            user=user,
            connection_string=hook.get_uri(),
            query_id=query_ids[0],
            session_id=session_id,
            key=self.key,
            history_window=self.history_window,
            query_history_result_limit=self.query_history_result_limit,
            retries=self.retries,
            retry_pause=self.retry_pause,
            raise_on_error=self.raise_on_error,
        )


def log_snowflake_resource_operator(op: SnowflakeOperator, **kwargs):
    task_id = kwargs.pop("task_id", "log_resources_%s" % (op.task_id))
    return LogSnowflakeResourceOperator(
        sql=op.sql,
        snowflake_conn_id=kwargs.pop("snowflake_conn_id", op.snowflake_conn_id),
        database=kwargs.pop("database", op.database),
        warehouse=kwargs.pop("warehouse", op.warehouse),
        role=kwargs.pop("role", op.role),
        schema=kwargs.pop("schema", op.schema),
        task_id=task_id,
        **kwargs
    )
